/*****************************************************************************

	Thermometer328_Mini
		
	Controllerboard mit einem ATMega328_Mini (gedrucken Schaltung) mit einem Temperatursensor 
	TSIG 306 ber eine One-Wire Verbindung angeschlossen.
	Der Sensor hat einen Temperaturbereich von -50 bis 150 Grad mit einer Genauigkeit 
	von +/- 0,3 Grand.
	Funktion:
	In der oberen LCD Anzeige erscheint die Temperatur, in der zweiten Zeile die Temperaturnderung
	der letzen Stunde.
	Nach dem Einschalten wird die aktuelle Spannung des Akkus im Display ausgegeben. Wenn < 3,3V,
	dann sollte nicht mehr eingeschaltet bleiben.

	Die weiteren Bedienelemente werden nicht verwendet. 

	Stromverbrauch: ca 0,6 mA, bei Messung jede Millisekunde 1,5 mA

	LED:		Lebenslicht blinkt mit 2 Hz, ausgeschaltet

	License:
    Copyright (C) 2016 Clemens Verstappen DL3ETW

	This program is free software; you can redistribute it and/or modify it under the terms  
 	of the GNU General Public License as published by the Free Software Foundation; either 
	version 3 of the License, or (at your option) any later version.	 
	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
  	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.You should have received a copy of 
	the GNU General Public License along with this program; if not, see <http://www.gnu.org/licenses/>. 

	Code for uart from Peter Fleury.
	http://homepage.hispeed.ch/peterfleury/avr-software.html

	history:
	V1.00 - first version Februar 2016

*****************************************************************************/
#include 	"lcd_drv.h"						//  LCD EADOG config
#include 	"LCD_DRV.C"
#include 	"i2cmaster.h"
#include 	<inttypes.h>                	// general lib
#include 	<avr/io.h>
#include 	<avr/interrupt.h>
#include 	<stdlib.h>						
#include 	<util/delay.h>	
#include 	"uart.h"
#include 	<avr/sleep.h>

			
#define 	UART_BAUD_RATE      9600      
#define 	F_CPU 				8000000UL 	// xtal frequency

#define		LEBENSLICHT 		PD2			// port pin for LED

#define		TIMER_100MS			0x030E		// 	time value for 100ms interrups of realtime_update 
#define		TIMER_200MS			0x061A		// 	200ms  
#define		TIMER_1S			0x1E83		//	1C4B		//  eine Sekunde


#define		SIGNAL_SENSOR 		PORTC2		//	Singnal Sensor
#define		POWER_SENSOR 		PORTC3		//	Power Sensor
#define 	Nop()  				asm volatile ("nop");

static 		volatile uint8_t 	i, j ,parity, strobe_time, strobe_count, index, tmax;
static 		volatile uint16_t	counter, temperatur, temp_int;
volatile 	uint8_t 			ad_channel;			// start value for display menue 
volatile 	float 				adc_result;
volatile 	float				t_slope;
volatile 	float 				history[60+1];		// Feld fr Messergebnisse, 4 Byte fr ein float oder double
volatile 	char 				ascii[16]; 			// string for lcd	
volatile 	char				ascii_uart[16];

volatile struct BitVariablen				// status of keys 
{									
	int		realtime_update :1;				// flag for main routine
}status = {1};


SIGNAL(TIMER1_COMPA_vect) 					// MAIN INTERVAL, start for main loop
{											// Signal(Timer_Compa_vect) Interrupt each 100 ms
	status.realtime_update = 1;				// indicate new calculation in main program
} 

uint16_t getTSicTemp();

// *************************************************************************

int main (void) 
{
//	DDRD 	|= (1<<LEBENSLICHT);	
	DDRC 	|= (1<<POWER_SENSOR);

	OCR1A	= TIMER_1S;									// timer 1, compare match
	TCCR1B	|= ((1 << WGM12) | (1 << CS12) | (1 << CS10)); 		// clear on compare match, divide by 1024
	TIMSK1	|= (1 << OCIE1A);							// timer1 compare match 1 interrupt enable

	SMCR 	|= (1<<SE);

	lcd_init();	
//	uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) );

	lcd_clear();										// welcome message on LCD
	lcd_pos( 0, 0 );
  	lcd_text( "  Thermometer   ");
	lcd_pos( 1, 0 );
  	lcd_text( "BY   DL3ETW     " );
	_delay_ms(500);

// ------------------------------------------------		// initialize ad converter and display battery voltage				

	ADMUX	|= (1 << REFS1) | (1 << REFS0);				// internal 1.1V referenz, 
	ADCSRA	|= (1 << ADEN) | (1 << ADSC) | (1 << ADATE) ;// enable AD, ADC Start Conversion,
	PORTC 	&= ~(1 << PC0);								// switch to input

	_delay_ms(1);										// 
 	adc_result = ADC;									// read second value
	adc_result = adc_result * 4.73 / 1024;				// with 100 k by 330 k use 4,73 ( here calibrated with DVM)
														// 	* 1,1 V * 1/ ( 100 k / (100 k + 330 k))  V / 1024 bit
														// use 1 % resistors. 
	dtostrf (adc_result, 2,2,(char*) ascii);			// zwo digits left of decimal, two right
	lcd_pos( 1,0);
  	lcd_text( "U-batt:         " );
	lcd_pos( 1,8);
	lcd_text((char*) ascii);
	lcd_pos( 1,13);
  	lcd_text("V");
	_delay_ms(500);
	lcd_pos( 1,0);
  	lcd_text( "                " );

	ADCSRA	&= ~(1 << ADEN);							// Stop AD Conversion,

// ------------------------------------------------ 

	sei();												// enable interrups
	counter = 0;
	tmax=0;
												
// ------------------------------------------------ 
  	while(1) 											// Endlosschleife
	{
		if (status.realtime_update)						// each 100ms work here
		{
			PORTD ^= (1<<LEBENSLICHT);					// toggle lebenslicht
			PORTC |= (1<<POWER_SENSOR);					// Schalte Sensor Power ein
			_delay_us(60);                          	// warten auf Einschwingen TSIC
			cli();										// Interrups aus
			history[1] = ((float)getTSicTemp()/ 2047 * 200) - 50;	// Temp-Sensor auslesen und ins Array ablegen
			sei(); 										// Interrups ein
			PORTC &= ~(1<<POWER_SENSOR);				// Schalte Sensor Power aus

			dtostrf (history[1], 3,1,(char*) ascii);	// float, three digits left of decimal, one right
			lcd_pos( 0,0);							
			lcd_text("Temp:           " );				// zeige aktuelle Temperatur an
			lcd_pos( 0,7);
			lcd_text((char*) ascii);
			lcd_pos( 0,12);
			lcd_data(223);
			lcd_pos( 0,13);								// ein Sonderzeichen Grad
			lcd_text("C");
			
			if ( counter == 60)							// alle  60 Sekunden ein Wert ins Feld 
			{
				tmax=tmax + 1;							// hierduch luft der tmax mit nach dem Einschalten
				if (tmax > 60)
					tmax = 60;
					
				for (index = 60; index >=1; index--)	// jede Minute alles im Feld eins Schieben 
					history[index+1] = history[index];

				t_slope = (history[1] - history[tmax]);	// rechne Differenz von vor einer Stunde aus

				dtostrf (t_slope, 3, 2, (char*) ascii);	// float, three digits left of decimal, one right
				lcd_pos( 1,0);							
				lcd_text("Slope:          " );			// zeige Temperaturnderung je Minute an
				lcd_pos( 1,7);
				lcd_text((char*) ascii);
				lcd_pos( 1,13);
				lcd_data(223);							// ein Sonderzeichen Grad
				lcd_pos( 1,14);
				lcd_text("/h");	

				counter = 0;
			}

			counter++;

		status.realtime_update = 0;	

		set_sleep_mode(SLEEP_MODE_IDLE);
       	sleep_mode();                   				// in den Schlafmodus wechseln

 	 	} // interrup close

	} // main while close
} // main close


uint16_t getTSicTemp(void)
{
    while(PINC & (1<<SIGNAL_SENSOR));           // warten auf high
    for (i=0;i<2;i++)                           // 2 Bytes Daten
    {
        strobe_time = 0;                        // fr Messung lschen
        while(PINC & (1<<SIGNAL_SENSOR));       // warten auf low
        										// Startbit, ab hier messen
        while(!( PINC & (1<<SIGNAL_SENSOR)))    // Low - Zeit messen
        {
            strobe_time++;                      // messen bitlnge
            if (strobe_time==0) return 999;     // berlauf, error
        }                                       // strobe_time hat ca. Wert 62 = 62s bei 40MHz
        parity = 0;
        for (j=0;j<8;j++)                       // 8 Datenbits
        {
            temperatur <<= 1;                   // nach links verschieben,
                                                // Bit 0 wird automatisch auf 0 gesetzt
            strobe_count = strobe_time;         // Zhler laden
            while(PINC & (1<<SIGNAL_SENSOR));   // auf fallende Flanke warten

            while(strobe_count--) { Nop(); }    // hier die gemessene Bitzeit warten

            if (PINC & (1<<SIGNAL_SENSOR))      // PIN messen und Bit 0 setzen wenn high
            {
                temperatur |= 0x01;
                parity++;                       // Paritt gleich mitzhlen
            }
            while(!( PINC & (1<<SIGNAL_SENSOR)));                  // weiter wenn high
        } 										// ende bitcount
        										// Parity Bit
        strobe_count = strobe_time;             // Zhler laden
        while(PINC & (1<<SIGNAL_SENSOR));       // auf Low warten

        while(strobe_count--) { Nop(); }        // hier die gemessene Bitzeit warten

        if (PINC & (1<<SIGNAL_SENSOR)) parity++;// Parity Bit dazurechnen

        if (parity % 2 != 0) return 999;        // Parity Fehler (muss gerade Zahl sein)

        while(!( PINC & (1<<SIGNAL_SENSOR)));   // weiter wenn high

    } 											// ende Bytecount
     return temperatur;							// Umrechnung in Temperatur, gleich in Festkomma mit einer Nachkommastelle
}



	// Belegung der Ports						Function			Signal name EAGLE Schaltplan Controler328_Mini
	// Port B Bit 0 PB0 (PCINT0/CKL0/ICP1)		key1				PB0_KEY1		
	//        Bit 1 PB1 (OC1A/PCINT1)			key2				PB1_KEY2
	//        Bit 2 PB2	(SS/OC1B/PCINT3)		DI	LCD				PB2_SI
  	//        Bit 3 PB3	(MOSI/OC2A/PCINT3))		MOSI / key3			PB3_SPI_MOSI_KEY3
  	//        Bit 4 PB4	(MISO/PCINT4))			MISO / key4			PB4_SPI_MISO_KEY4
  	//        Bit 5 PB5 (SCK/PCINT5)			SCK  / encoder_key	PB5_SPI_SCK_KEY5
  	//        Bit 6 PB6	(PCINT6/XTAL1/TOSC1)	8MHz X-TAL			PB6
	//		  Bit 7 PB7	(PCINT7/XTAL2/TOSC2)	8MHz X-TAL			PB7

 	// Port C Bit 0 PC0 (ADC0/PCINT8)			Voltage Akku		PC0_ADC0
  	//        Bit 1 PC1	(ADC1/PCINT9)			-					PC1_ADC1	
 	//        Bit 2 PC2	(ADC2/PCINT10)			SIGNAL_SENSOR		PC2_ADC2	
  	//        Bit 3 PC3	(ADC3/PCINT11)			POWER_SENSOR		PC3_ADC3
  	//        Bit 4 PC4	(ADC4/SDA/PCINT12)		SDA					PC4_SDA
  	//        Bit 5 PC5	(ADC5/SCL/PCINT13)		SCL					PC5_SCL
  	//        Bit 6 PC6	(PCINT/RESET)			Reset
  			          										
	// Port D Bit 0 PD0	(PCINT16/RXD)			-					PD0_RX			
	//        Bit 1 PD1	(PCINT17/TXD)			-					PD1_TX
  	//        Bit 2 PD2	(PCINT18/INT0)			Lebenslicht			PD2_SI
  	//        Bit 3 PD3	(PCINT19/OC2B/INT1)		Encoder A			PD3_ENCODERA
  	//        Bit 4 PD4	(PCINT20/XCK/T0)		RS LCD				PD4_RS
  	//        Bit 5 PD5	(PCINT21/OC0B/T1)		Encoder B			PD5_ENCODERB
  	//        Bit 6 PD6	(PCINT22/OC0A/AIN0)		CSB	LCD				PD6_CSB
  	//        Bit 7 PD7	(PCINT23/AIN1)			SCK	LCD				PD7_SCK

/*			for (index = 60; index >=1; index--)		// Gebe Feldinhalt ber die serielle aus 	
				{
					itoa(index,(char*) ascii_uart,10);
					uart_puts("   index: ");			// UART MELDUNG
					uart_puts((char*) ascii_uart);
					uart_puts("    ");					// UART MELDUNG

					dtostrf (history[index], 3,1,(char*) ascii);	// float, three digits left of decimal, one right
					uart_puts("float: ");				// UART MELDUNG
					uart_puts((char*) ascii);

					uart_putc(10);						// noch ein Linefeed und cr hinterher
					uart_putc(13);
				}	
*/					
